/* Copyright (c) 2007-2012 Sam Trenholme
 *
 * TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This software is provided 'as is' with no guarantees of correctness or
 * fitness for purpose.
 */

/* Some constants pointing to mararc parameters */

/* mararc string parameters */
#define DWM_S_bind_address 0
#define DWM_S_ipv4_bind_addresses 1
#define DWM_S_chroot_dir 2
#define DWM_S_recursive_acl 3
#define DWM_S_random_seed_file 4
#define DWM_S_cache_file 5

/* mararc dictionary parameters */
#define DWM_D_upstream_servers 0

/* mararc numeric parameters */
#define DWM_N_maxprocs 0
#define DWM_N_timeout_seconds 1
#define DWM_N_dns_port 2
#define DWM_N_upstream_port 3
#define DWM_N_handle_overload 4
#define DWM_N_handle_noreply 5
#define DWM_N_recurse_min_bind_port 6
#define DWM_N_recurse_number_ports 7
#define DWM_N_hash_magic_number 8
#define DWM_N_maximum_cache_elements 9
#define DWM_N_maradns_uid 10
#define DWM_N_maradns_gid 11
#define DWM_N_resurrections 12
#define DWM_N_num_retries 13
#define DWM_N_verbose_level 14
#define DWM_N_deliver_all 15
#define DWM_N_max_ttl 16

/* Various character classes used by the Mararc parser's finite state
 * machine */

#define dwm_is_alpha(c)      (c >= 'a' && c <= 'z') || \
                             (c >= 'A' && c <= 'Z') || \
                             c == '_'

#define dwm_is_alphanum(c)   (c >= 'a' && c <= 'z') || \
                             (c >= 'A' && c <= 'Z') || \
                             (c >= '0' && c <= '9') || \
                             c == '_'

#define dwm_is_alphastart(c) (c >= 'a' && c <= 'z') || \
                             (c >= 'A' && c <= 'Z')

#define dwm_is_dname(c)      (c >= 'a' && c <= 'z') || \
                             (c >= 'A' && c <= 'Z') || \
                             (c >= '0' && c <= '9') || \
                             c == '-' || c == '.' || c == '_'

#define dwm_is_instring(c)   (c >= ' ' && c <= '~' && c != '#' && c != '"')

#define dwm_is_whitespace(c) (c == ' ' || c == '\t')

#define dwm_is_any(c)        (c >= ' ' && c <= '~') || c == '\t' || c > 127

#define dwm_is_dnamestart(c) (c >= 'a' && c <= 'z') || \
                             (c >= 'A' && c <= 'Z') || \
                             (c >= '0' && c <= '9')

#define dwm_is_number(c)     (c >= '0' && c <= '9')

/* Limits to the number of states, actions, and pattern per state we can
 * have */

/* Maximum number of states */
#define DWM_MAX_STATES 52
/* Maximum number of patterns per state */
#define DWM_MAX_PATTERNS 8

/* The actual state machine that we use to parse a MaraRC file; this is
 * described in the file doc/internals/MARARC.parser */

#define dwm_machine "a Hb Y1c Wa Rxp T;\n" \
                    "b Xb Rxp T;\n" \
                    "c B1c Wd =e [f +g\n" \
                    "d Wd =e [f +g\n" \
                    "e We N4h Qi {6w\n" \
                    "f Wf Qn\n" \
                    "g =5e\n" \
                    "h N4h Wk Hb Rxp T;\n" \
                    "i I3m\n" \
                    "k Wk Hb Rxp T;\n" \
                    "m I3m Qk\n" \
                    "n .2o\n" \
                    "o Qq\n" \
                    "q Wq ]r\n" \
                    "r Wr =s +t\n" \
                    "s Ws Qu\n" \
                    "t =5s\n" \
                    "u I3v\n" \
                    "v I3v Qk\n" \
                    "w }k\n" \
                    "xp T;\n"

/* A tokenized single state in the finite state machine */
typedef struct {
        char pattern[DWM_MAX_PATTERNS + 1];
        char action[DWM_MAX_PATTERNS + 1];
        char newstate[DWM_MAX_PATTERNS + 1];
        } dwm_fs;

/* Parse a mararc file; this should only be called once when executing
 * deadwood.  Note that this is the *only* public method in this entire
 * file; all other functions in this file should only be called from
 * other functions in this file.
 * Input: c string that points to mararc file
 * Output: 1 on success; program exits on mararc parse error */
int dwm_parse_mararc(char *name);

